# -*- coding: utf-8 -*-
from json import loads as _parseJSON
from urllib.request import urlopen as _urlopen
from subprocess import getstatusoutput as _getstatusoutput
import re as _re
import json

from npm2deb import Npm2Deb as _Npm2Deb
from npm2deb.utils import debug as _debug, debianize_name as _debianize_name
from npm2deb.mapper import Mapper as _Mapper


DO_PRINT = False


def my_print(what):
    if DO_PRINT:
        print(what)


def search_for_repository(module):
    if isinstance(module, _Npm2Deb):
        module = module.debian_name
    else:
        module = 'node-%s' % _debianize_name(module)

    formatted = "  {0:40} -- {1}"
    found = False
    result = {}

    my_print("Looking for existing repositories on salsa.debian.org:")
    data = json.loads(_urlopen(
        "https://salsa.debian.org/groups/js-team/-/children.json?filter=%s" % module).read())
    if len(data) > 0:
        found = True
        for repo in data:
            name = repo['name']
            description = repo['description']
            result[name] = description
            my_print(formatted.format(name, description))
    if not found:
        my_print("  None")
    return result


def search_for_bug(module):
    if isinstance(module, _Npm2Deb):
        module = module.debian_name
    else:
        module = 'node-%s' % _debianize_name(module)

    my_print('Looking for wnpp bugs:')
    _debug(1, "calling wnpp-check")
    info = _getstatusoutput('wnpp-check %s' % module)
    if info[0] == 0:
        my_print('  None')
        return []
    else:
        lines = info[1].split('\n')
        formatted = "  #{0}  {1:>3}:  {2:25} -- {3}"
        result = []
        for line in lines:
            try:
                bug = {}
                match = _re.match('\\((.*) - #(\\d+).*\\) (.*) (.*)$', line)
                bug['num'] = match.group(2)
                bug['type'] = match.group(1)
                bug['package'] = match.group(4)
                bug['url'] = match.group(3)
                result.append(bug)
                my_print(formatted.format(bug["num"],
                                          bug["type"],
                                          bug["package"],
                                          bug["url"]))
            except:
                continue
        return result


def search_for_reverse_dependencies(module):
    if isinstance(module, _Npm2Deb):
        module = module.name
    url = "https://skimdb.npmjs.com/registry/_design/app/_view/dependedUpon?startkey=" \
        + "[%%22%(name)s%%22]&endkey=[%%22%(name)s%%22,%%7B%%7D]&group_level=2"
    url = url % {'name': module}
    _debug(1, "opening url %s" % url)
    data = _urlopen(url).read().decode('utf-8')
    data = _parseJSON(data)
    result = []
    if 'rows' in data and len(data['rows']) > 0:
        my_print("Reverse Depends:")
        for row in data['rows']:
            dependency = row['key'][1]
            result.append(dependency)
            my_print("  %s" % dependency)
    else:
        my_print("Module %s has no reverse dependencies" % module)
    return result


def search_for_dependencies(module, recursive=False, force=False,
                            prefix=u'', expanded_dependencies=[]):
    try:
        if not isinstance(module, _Npm2Deb):
            _debug(1, 'getting dependencies - calling npm view %s' % module)
            npm_out = _getstatusoutput('npm view "%s" '
                                       'dependencies --json 2>/dev/null'
                                       % module)[1]
            dependencies = _parseJSON(npm_out)
        else:
            dependencies = module.json['dependencies']
            module = module.name
    except ValueError:
        return None

    mapper = _Mapper.get_instance()
    result = {}

    keys = sorted(list(dependencies.keys()))
    last_dep = False
    for dep in keys:
        if dep == keys[-1]:
            last_dep = True
        result[dep] = {}
        result[dep]['version'] = dependencies[dep]
        result[dep]['debian'] = mapper.get_debian_package(dep)['repr']
        dep_prefix = u'└─' if last_dep else u'├─'
        print_formatted_dependency(u"%s %s (%s)" % (dep_prefix, dep,
                                                    result[dep]['version']),
                                   result[dep]['debian'], prefix)
        if recursive and dep not in expanded_dependencies:
            expanded_dependencies.append(dep)
            if (result[dep]['debian'] and force) or \
                    result[dep]['debian'] is None:
                new_prefix = "%s%s " % (prefix, u"  " if last_dep else u"│ ")
                result[dep]['dependencies'] = \
                    search_for_dependencies(dep, recursive, force, new_prefix,
                                            expanded_dependencies)
        else:
            result[dep]['dependencies'] = None

    return result


def search_for_builddep(module):
    try:
        if not isinstance(module, _Npm2Deb):
            _debug(1, 'getting builddep - calling npm view %s' % module)
            npm_out = _getstatusoutput('npm view "%s" '
                                       'devDependencies --json 2>/dev/null'
                                       % module)[1]
            builddeb = _parseJSON(npm_out)
        else:
            builddeb = module.json['devDependencies']
            module = module.name
    except ValueError:
        return None

    mapper = _Mapper.get_instance()
    result = {}

    for dep in sorted(builddeb.keys()):
        result[dep] = {}
        result[dep]['version'] = builddeb[dep]
        result[dep]['debian'] = mapper.get_debian_package(dep)['repr']
        print_formatted_dependency("%s (%s)" % (dep, result[dep]['version']),
                                   result[dep]['debian'])

    return result


def print_formatted_dependency(npm, debian, prefix=u''):
    formatted = u"{0:50}{1}"
    my_print(formatted.format(u"%s%s" % (prefix, npm), debian))



from argparse import ArgumentParser as _ArgumentParser
import os as _os
import sys as _sys

from npm2deb import Npm2Deb as _Npm2Deb
from npm2deb import utils as _utils
from npm2deb import templates as _templates
from npm2deb import helper as _helper
from npm2deb import Mapper as _Mapper
import npm2deb as _


def main(argv=None):
    # verify utf8 support
    _utils.verify_python3_env()
    if not argv:
        argv = _sys.argv
    parser = _ArgumentParser(prog='npm2deb')
    parser.add_argument('-D', '--debug', type=int, help='set debug level')
    parser.add_argument(
        '-v', '--version', action='version', version='%(prog)s ' + _.VERSION)

    subparsers = parser.add_subparsers(title='commands')

    parser_create = subparsers.add_parser(
        'create', help='create the debian files')
    parser_create.add_argument(
        '-n',
        '--noclean',
        action="store_true",
        default=False,
        help='do not remove files downloaded with npm')
    parser_create.add_argument(
        '--debhelper',
        default=_.DEBHELPER,
        help='specify debhelper version [default: %(default)s]')
    parser_create.add_argument(
        '--standards-version',
        default=_utils.get_latest_debian_standards_version(),
        help='set standards-version [default: %(default)s]')
    parser_create.add_argument(
        '--upstream-author',
        default=None,
        help='set upstream author if not automatically recognized')
    parser_create.add_argument(
        '--upstream-homepage',
        default=None,
        help='set upstream homepage if not automatically recognized')
    parser_create.add_argument(
        '--upstream-license',
        default=None,
        help='set upstream license if not automatically recognized')
    parser_create.add_argument(
        '--debian-license',
        default=None,
        help='license used for debian files [default: the same of upstream]')
    parser_create.add_argument(
        'node_module', help='node module available via npm')
    parser_create.set_defaults(func=create)

    parser_view = subparsers.add_parser(
        'view', help="a summary view of a node module")
    parser_view.add_argument(
        '-j',
        '--json',
        action="store_true",
        default=False,
        help='print verbose information in json format')
    parser_view.add_argument(
        'node_module', help='node module available via npm')
    parser_view.set_defaults(func=print_view)

    parser_depends = subparsers.add_parser(
        'depends', help='show module dependencies in npm and debian')
    parser_depends.add_argument(
        '-r',
        '--recursive',
        action="store_true",
        default=False,
        help='look for binary dependencies recursively')
    parser_depends.add_argument(
        '-f',
        '--force',
        action="store_true",
        default=False,
        help='force inspection for modules already in debian')
    parser_depends.add_argument(
        '-b',
        '--binary',
        action="store_true",
        default=False,
        help='show binary dependencies')
    parser_depends.add_argument(
        '-B',
        '--builddeb',
        action="store_true",
        default=False,
        help='show build dependencies')
    parser_depends.add_argument(
        'node_module', help='node module available via npm')
    parser_depends.set_defaults(func=show_dependencies)

    parser_rdepends = subparsers.add_parser(
        'rdepends', help='show the reverse dependencies for module')
    parser_rdepends.add_argument(
        'node_module', help='node module available via npm')
    parser_rdepends.set_defaults(func=show_reverse_dependencies)

    parser_search = subparsers.add_parser(
        'search', help="look for module in debian project")
    parser_search.add_argument(
        '-b',
        '--bug',
        action="store_true",
        default=False,
        help='search for existing bug in wnpp')
    parser_search.add_argument(
        '-d',
        '--debian',
        action="store_true",
        default=False,
        help='search for existing package in debian')
    parser_search.add_argument(
        '-r',
        '--repository',
        action="store_true",
        default=False,
        help='search for existing repository in alioth')
    parser_search.add_argument(
        'node_module', help='node module available via npm')
    parser_search.set_defaults(func=search_for_module)

    parser_itp = subparsers.add_parser('itp', help="print a itp bug template")
    parser_itp.add_argument(
        'node_module', help='node module available via npm')
    parser_itp.set_defaults(func=print_itp)

    parser_license = subparsers.add_parser(
        'license', help='print license template and exit')
    parser_license.add_argument(
        '-l',
        '--list',
        action="store_true",
        default=False,
        help='show the available licenses')
    parser_license.add_argument(
        'name', nargs='?', help='the license name to show')
    parser_license.set_defaults(func=print_license)

    if len(argv) == 1:
        parser.error("Please specify an option.")
    else:
        args = parser.parse_args(argv[1:])
        if args.debug:
            _utils.DEBUG_LEVEL = args.debug

        args.func(args)


def search_for_module(args):
    _helper.DO_PRINT = True
    # enable all by default
    if not (args.bug or args.debian or args.repository):
        args.bug = True
        args.debian = True
        args.repository = True
    node_module = get_npm2deb_instance(args).name
    if args.debian:
        print("\nLooking for similar package:")
        mapper = _Mapper.get_instance()
        pkg_info = mapper.get_debian_package(node_module)
        print("  %s (%s)" % (pkg_info['repr'], pkg_info['suite']))
    if args.repository:
        print("")
        _helper.search_for_repository(node_module)
    if args.bug:
        print("")
        _helper.search_for_bug(node_module)
    print("")

    _show_mapper_warnings()


def print_view(args):
    npm2deb_instance = get_npm2deb_instance(args)
    if args.json:
        from json import dumps
        # print a clean version of json module
        json = npm2deb_instance.json
        for key in [
                'time', 'versions', 'dist-tags', 'component', 'users', 'time',
                'maintainers', 'readmeFilename', 'contributors', 'keywords'
        ]:
            if key in json:
                del json[key]
        print(dumps(json, indent=4, sort_keys=True))

    else:
        formatted = "{0:40}{1}"
        for key in ['name', 'version', 'description', 'homepage', 'license']:
            attr_key = key
            if key == 'license' or key == 'version':
                attr_key = 'upstream_%s' % key
            print(
                formatted.format("%s:" % key.capitalize(),
                                 getattr(npm2deb_instance, attr_key, None)))

        mapper = _Mapper.get_instance()
        pkg_info = mapper.get_debian_package(npm2deb_instance.name)
        print(
            formatted.format(
                "Debian:", "%s (%s)" % (pkg_info['repr'], pkg_info['suite'])))

        if mapper.has_warnings():
            print("")
            _show_mapper_warnings()


def print_itp(args):
    print(get_npm2deb_instance(args).get_ITP())


def print_license(args, prefix=""):
    if args.list:
        licenses = sorted(_templates.LICENSES.keys())
        print("%s Available licenses are: %s." % (prefix,
                                                  ', '.join(licenses).lower()))
    else:
        if args.name is None:
            print("You have to specify a license name")
            args.list = True
            print_license(args)
        else:
            template_license = _utils.get_license(args.name)
            if not template_license.startswith('FIX_ME'):
                print(template_license)
            else:
                print("Wrong license name.")
                args.list = True
                print_license(args)


def show_dependencies(args):
    _helper.DO_PRINT = True
    # enable all by default
    if not args.binary and not args.builddeb:
        args.binary = True
        args.builddeb = True

    npm2deb_instance = get_npm2deb_instance(args)
    module_name = npm2deb_instance.name
    json = npm2deb_instance.json

    if args.binary:
        if 'dependencies' in json and json['dependencies']:
            print("Dependencies:")
            _helper.print_formatted_dependency("NPM", "Debian")
            module_ver = npm2deb_instance.upstream_version
            module_deb = _Mapper.get_instance()\
                .get_debian_package(module_name)["repr"]
            _helper.print_formatted_dependency(
                "%s (%s)" % (module_name, module_ver), module_deb)
            _helper.search_for_dependencies(npm2deb_instance, args.recursive,
                                            args.force)
            print("")
        else:
            print("Module %s has no dependencies." % module_name)

    if args.builddeb:
        if 'devDependencies' in json and json['devDependencies']:
            print("Build dependencies:")
            _helper.print_formatted_dependency("NPM", "Debian")
            _helper.search_for_builddep(npm2deb_instance)
            print("")
        else:
            print("Module %s has no build dependencies." % module_name)

    _show_mapper_warnings()


def show_reverse_dependencies(args):
    _helper.DO_PRINT = True
    node_module = get_npm2deb_instance(args).name
    _helper.search_for_reverse_dependencies(node_module)


def create(args):
    npm2deb = get_npm2deb_instance(args)
    try:
        saved_path = _os.getcwd()
        _utils.create_dir(npm2deb.name)
        _utils.change_dir(npm2deb.name)
        npm2deb.start()
        _utils.change_dir(npm2deb.debian_name)
        npm2deb.initiate_build(saved_path)

    except OSError as os_error:
        print(str(os_error))
        exit(1)

    _show_mapper_warnings()


def get_npm2deb_instance(args):
    if not args.node_module or len(args.node_module) is 0:
        print('please specify a node_module.')
        exit(1)
    try:
        return _Npm2Deb(args=vars(args))
    except ValueError as value_error:
        print(value_error)
        exit(1)


def _show_mapper_warnings():
    mapper = _Mapper.get_instance()
    if mapper.has_warnings():
        print("Warnings occurred:")
        mapper.show_warnings()
        print("")


if __name__ == '__main__':
    main()



CHANGELOG = """%(debian_name)s (%(version)s-1) UNRELEASED; urgency=low

  * Initial release (Closes: #nnnn)

 -- %(debian_author)s  %(date)s
"""

description_template = """ Write the short and long descriptions for the Debian package as
 explained in the Developer's Reference, §6.2.1 – §6.2.3.
 .
 You can start with the short upstream package description,
 “%(upstream_description)s”.
 .
 Be aware that most upstream package descriptions are not written to
 conform with Debian package guidelines. You need to explain the role
 of this package for a Debian audience.
 .
 Node.js is an event-based server-side JavaScript engine.
"""

CONTROL = """Source: %(Source)s
Section: javascript
Priority: optional
Maintainer: Debian Javascript Maintainers <pkg-javascript-devel@lists.alioth.debian.org>
Uploaders: %(Uploaders)s
Testsuite: autopkgtest-pkg-nodejs
Build-Depends:
 debhelper (>= %(debhelper)s)
 , nodejs (>= 6)
 , pkg-js-tools
Standards-Version: %(Standards-Version)s
Homepage: %(Homepage)s
Vcs-Git: %(Vcs-Git)s
Vcs-Browser: %(Vcs-Browser)s

Package: %(Package)s
Architecture: all
Depends:
 ${misc:Depends}
 , %(Depends)s
Description: %(Description)s
""" + description_template

RULES = """#!/usr/bin/make -f
# -*- makefile -*-

# Uncomment this to turn on verbose mode.
#export DH_VERBOSE=1

%%:
	dh $@ --with nodejs

#override_dh_auto_build:

%(overrides)s"""

COPYRIGHT = """Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: %(upstream_name)s
Upstream-Contact: %(upstream_contact)s
Source: %(source)s

Files: *
Copyright: %(upstream_date)s, %(upstream_author)s
License: %(upstream_license_name)s
%(upstream_license)s
Files: debian/*
Copyright: %(debian_date)s, %(debian_author)s
License: %(debian_license_name)s

%(debian_license)s"""

WNPP = """To: submit@bugs.debian.org
Subject: ITP: %(debian_name)s -- %(description)s

Package: wnpp
Severity: wishlist
Owner: %(debian_author)s
X-Debbugs-CC: debian-devel@lists.debian.org

* Package name    : %(debian_name)s
  Version         : %(version)s
  Upstream Author : %(upstream_author)s
* URL             : %(homepage)s
* License         : %(license)s
  Programming Lang: JavaScript
  Description     : %(description)s

 FIX_ME: This ITP report is not ready for submission, until you are
 confident this package description is ready for Debian.
 .
""" + description_template + """
FIX_ME: Explain why this package is suitable for adding to Debian. Is
it a dependency of some other package? What benefit does it provide
compared to other similar packages already in Debian?

FIX_ME: Explain how you intend to consistently maintain this package
in Debian. If you are not yet a Debian member, does this package need
a sponsor? Do you have co-maintainers? Are you a member of the Debian
JavaScript maintainers team?
"""

LICENSES = {}

LICENSES['GPL-2'] = """GPL-2+
 This package is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 .
 This package is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 .
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>
 .
 On Debian systems, the complete text of the GNU General
 Public License version 2 can be found in "/usr/share/common-licenses/GPL-2".
"""

LICENSES['GPL-3'] = """GPL-3+
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 .
 This package is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 .
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.
 .
 On Debian systems, the complete text of the GNU General
 Public License version 3 can be found in "/usr/share/common-licenses/GPL-3".
"""

LICENSES['Apache'] = """Apache-2.0
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 .
 http://www.apache.org/licenses/LICENSE-2.0
 .
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 .
 On Debian systems, the complete text of the Apache version 2.0 license
 can be found in "/usr/share/common-licenses/Apache-2.0".
"""

LICENSES['Artistic'] = """Artistic-1.0
 This program is free software; you can redistribute it and/or modify it
 under the terms of the "Artistic License" which comes with Debian.
 .
 THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 .
 On Debian systems, the complete text of the Artistic License
 can be found in "/usr/share/common-licenses/Artistic".
"""

LICENSES['BSD-2-clause'] = """%(name)s
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 .
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE HOLDERS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

LICENSES['BSD-3-clause'] = """%(name)s
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the University nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.
 .
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE HOLDERS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

LICENSES['BSD-4-clause'] = """%(name)s
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the University nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.
 4. Neither the name of the organization nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.
 .
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE HOLDERS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

LICENSES['LGPL-2'] = """LGPL-2
 This package is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 .
 This package is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 .
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.
 .
 On Debian systems, the complete text of the GNU Lesser General
 Public License can be found in "/usr/share/common-licenses/LGPL-2".
"""

LICENSES['LGPL-3'] = """LGPL-3
 This package is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 3 of the License, or (at your option) any later version.
 .
 This package is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 .
 You should have received a copy of the GNU General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.
 .
 On Debian systems, the complete text of the GNU Lesser General
 Public License can be found in "/usr/share/common-licenses/LGPL-3".
"""

LICENSES['Expat'] = """Expat
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation files
 (the "Software"), to deal in the Software without restriction,
 including without limitation the rights to use, copy, modify, merge,
 publish, distribute, sublicense, and/or sell copies of the Software,
 and to permit persons to whom the Software is furnished to do so,
 subject to the following conditions:
 .
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
 .
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
"""

LICENSES['MIT'] = LICENSES['Expat']

LICENSES['ISC'] = """ISC
 Permission to use, copy, modify, and/or distribute this software for any
 purpose with or without fee is hereby granted, provided that the above
 copyright notice and this permission notice appear in all copies.
 .
 THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
"""

WATCH = {}

WATCH['github'] = """version=4
opts=\\
dversionmangle=%(dversionmangle)s,\\
filenamemangle=s/.*\/v?([\d\.-]+)\.tar\.gz/%(debian_name)s-$1.tar.gz/ \\
 %(url)s/releases .*/archive/v?([\d\.]+).tar.gz
"""

WATCH['gitlab'] = """version=4
opts=\\
dversionmangle=%(dversionmangle)s \\
 %(url)s/tags?sort=updated_desc .*/archive/.*?v?([\d\.]+)\.tar\.gz
"""

WATCH['npmregistry'] = """version=4
# It is not recommended use npmregistry. Please investigate more.
# Origin url: %(url)s
# Take a look at https://wiki.debian.org/debian/watch/
opts="searchmode=plain,pgpmode=none" \\
 https://registry.npmjs.org/%(module)s \\
 https://registry.npmjs.org/%(remodule)s/-/%(modulename)s-(\d[\d\.]*)@ARCHIVE_EXT@
"""

METADATA = {}

METADATA['github'] = """---
Archive: GitHub
Bug-Database: %(url)s/issues
Contact: %(url)s/issues
Name: %(module)s
Repository: %(url)s.git
Repository-Browse: %(url)s
"""

METADATA['gitlab'] = """---
Archive: GitLab
Bug-Database: %(url)s/issues
Contact: %(url)s/issues
Name: %(module)s
Repository: %(url)s.git
Repository-Browse: %(url)s
"""

GBPCONF = """[DEFAULT]
pristine-tar = True
filter = [ '.gitignore', '.travis.yml', '.git*' ]
"""



from json import loads as _parseJSON
from re import findall as _findall
from urllib.request import urlopen as _urlopen
from fnmatch import fnmatch as _fnmatch

import apt_pkg

from npm2deb.utils import debug as _debug
from npm2deb.utils import debianize_name as _debianize_name

DB_URL = 'https://wiki.debian.org/Javascript/Nodejs/Database'
MADISON_URL = 'https://api.ftp-master.debian.org/madison?S=true&f=python&package=%s'


class Mapper(object):
    INSTANCE = None

    def __init__(self):
        if self.INSTANCE is not None:
            raise ValueError("Mapper is a Singleton. "
                             "Please use get_instance method.")
        _debug(2, 'loading database from %s' % DB_URL)
        data = _findall(
            '{{{(.*)}}}',
            _urlopen("%s?action=raw" % DB_URL).read().decode('utf-8').replace(
                '\n', ''))[0]
        self.json = _parseJSON(data)
        self._warnings = {}
        self.reset_warnings()
        apt_pkg.init()

    @classmethod
    def get_instance(cls):
        if cls.INSTANCE is None:
            cls.INSTANCE = Mapper()
        return cls.INSTANCE

    def get_debian_package(self, node_module):
        result = {}
        result['info'] = None
        result['name'] = None
        result['version'] = None
        result['suite'] = None
        result['repr'] = None
        db_package = None

        if node_module in self.json:
            db_package = self.json[node_module]
        else:
            for pattern in self.json.keys():
                if _fnmatch(node_module, pattern):
                    db_package = self.json[pattern]
                    break

        if db_package:
            if 'replace' in db_package:
                result['name'] = db_package['replace']
            if 'info' in db_package:
                result['info'] = ('info', db_package['info'])
                self.append_warning('info', node_module, db_package['info'])
            elif 'warning' in db_package:
                result['info'] = ('warning', db_package['warning'])
                self.append_warning('warning', node_module,
                                    db_package['warning'])
            elif 'error' in db_package:
                result['info'] = ('error', db_package['error'])
                self.append_warning('error', node_module, db_package['error'])
        else:
            result['name'] = 'node-%s' % _debianize_name(node_module)

        if not result['name']:
            return result

        data = _urlopen(MADISON_URL % result['name']).read().decode('utf-8')
        packages = _parseJSON(data)
        if len(packages) < 1:
            result['name'] = None
            return result
        result['version'] = '0:0'
        for suite, versions in packages[0][result['name']].items():
            for version, source in versions.items():
                if apt_pkg.version_compare(version, result['version']) > 0:
                    result['version'] = version
                    result['suite'] = suite
                    result['name'] = source['source']
                    result['repr'] = '%s (%s)' % (result['name'],
                                                  result['version'])
        result['version'] = result['version']
        if result['version'] == '0:0':
            result['version'] = None

        return result

    def get_warnings(self, reset=True):
        result = self._warnings
        if reset:
            self.reset_warnings()
        return result

    def show_warnings(self, reset=True):
        for label in ['info', 'warning', 'error']:
            for name in self._warnings[label]:
                self._print_formatted_warning(label, name)
        if reset:
            self.reset_warnings()

    def has_warnings(self):
        if self._warnings['info'] or self._warnings['error'] or \
                self._warnings['warning']:
            return True
        return False

    def reset_warnings(self):
        self._warnings = {}
        self._warnings['info'] = {}
        self._warnings['error'] = {}
        self._warnings['warning'] = {}

    def append_warning(self, label, node_module, warning):
        self._warnings[label][node_module] = warning

    def _print_formatted_warning(self, label, name):
        formatted = ' {0:9} {1}: {2}'
        msg = self._warnings[label][name]
        print(formatted.format("[%s]" % label, name, msg))



#!/usr/bin/python3

import sys
import os
import inspect
current = os.path.dirname(os.path.abspath(
                          inspect.getfile(
                              inspect.currentframe()) + '/..'))
sys.path.append(current)

import unittest
from shutil import rmtree
from npm2deb import Npm2Deb, DEBHELPER


class npm2deb_fails(unittest.TestCase):
    def test_init_no_npm_module(self):
        try:
            Npm2Deb('MODULE_NOT_IN_NPM')
            raise Exception
        except Exception as err:
            self.assertTrue(isinstance(err, ValueError))
            # must suggest type of failure
            self.assertTrue(str(err).find('npm reports errors about') >= 0)

    def test_init_module_multiple_version(self):
        try:
            Npm2Deb('socket.io@0.9.x')
            raise Exception
        except Exception as err:
            self.assertTrue(isinstance(err, ValueError))
            # must suggest type of failure
            self.assertTrue(str(err)
                            .find('More than one version found. '
                                  'Please specify one of:') >= 0)


class npm_coherences_license(unittest.TestCase):

    def test_upstream_license_as_str(self):
        n = Npm2Deb('parse-url')
        self.assertEqual(n.upstream_license, 'Expat')

    def test_upstream_license_as_dict(self):
        n = Npm2Deb('utils-merge')
        self.assertEqual(n.upstream_license, 'Expat')

    def test_upstream_license_as_list(self):
        n = Npm2Deb('amdefine')
        self.assertEqual(n.upstream_license, 'BSD')

    def test_upstream_license_as_argument(self):
        n = Npm2Deb('amdefine', {'upstream_license': 'MIT'})
        self.assertEqual(n.upstream_license, 'MIT')

    def test_upstream_license_equals_debian_license(self):
        n = Npm2Deb('parse-url')
        self.assertEqual(n.upstream_license, n.debian_license)

    def test_debian_license_as_argument(self):
        n = Npm2Deb('amdefine', {'debian_license': 'GPL-3'})
        self.assertEqual(n.debian_license, 'GPL-3')


class read_json(unittest.TestCase):

    def test_json_via_npm(self):
        n = Npm2Deb('serve-index')
        self.assertEqual(n.name, 'serve-index')

    def test_json_via_url(self):
        n = Npm2Deb('https://raw.githubusercontent.com/' +
                    'expressjs/serve-index/master/package.json')
        self.assertEqual(n.name, 'serve-index')

    def test_json_via_file(self):
        file_name = '/tmp/test_json_via_file'
        test_name = "test_name"
        with open(file_name, 'w') as fd:
            fd.write('{"name":"%s"}' % test_name)
        n = Npm2Deb(file_name)
        os.remove(file_name)
        self.assertEqual(n.name, test_name)


class debian(unittest.TestCase):

    def tearDown(self):
        if os.path.isdir('debian'):
            rmtree('debian')

    def _get_compat(self):
        with open('debian/compat', 'r') as compat_fd:
            return compat_fd.read().strip('\n')

    def _get_file_line(self, filename, what):
        result = None
        with open(filename, 'r') as file_fd:
            for line in file_fd.readlines():
                if line.find(what) >= 0:
                    result = line.strip('\n')
                    break
        return result

    def _get_debfile_line(self, debian_filename, what):
        return self._get_file_line('debian/%s' % debian_filename, what)

    def test_debhelper(self):
        n = Npm2Deb('parse-url')
        n.create_base_debian()
        n.create_control()
        self.assertEqual(self._get_compat(), str(DEBHELPER))
        self.assertEqual(self._get_debfile_line("control",
                                                " debhelper ("), ' debhelper (>= %s)' % DEBHELPER)

    def test_debhelper_as_argument(self):
        MY_DEBHELPER = DEBHELPER + 1
        n = Npm2Deb('parse-url', {'debhelper': MY_DEBHELPER})
        n.create_base_debian()
        n.create_control()
        self.assertEqual(self._get_compat(), str(MY_DEBHELPER))
        self.assertEqual(self._get_debfile_line("control", " debhelper ("),
                         ' debhelper (>= %s)' % MY_DEBHELPER)

    def test_manpages(self):
        n = Npm2Deb('jade')
        n.create_base_debian()
        n.create_manpages()
        self.assertEqual(self._get_debfile_line(
            'manpages', 'jade.1'), "jade.1")

    def test_watch_github(self):
        n = Npm2Deb('serve-static')
        n.create_base_debian()
        n.create_watch()
        line = self._get_debfile_line('watch', '/tags')
        self.assertTrue(line is not None and len(line) > 0)

    def test_watch_npmregistry(self):
        # must create a npmregistry since we do not know about git url
        n = Npm2Deb('yg-panache')
        n.create_base_debian()
        n.create_watch()
        line = self._get_debfile_line('watch', '/npmregistry')
        self.assertTrue(line is not None and len(line) > 0)

    def test_watch_github_with_no_tags(self):
        # must fallback on npmregistry if no tags in github
        n = Npm2Deb('security')
        n.create_base_debian()
        n.create_watch()
        line = self._get_debfile_line('watch', '/npmregistry')
        self.assertTrue(line is not None and len(line) > 0)

    def test_repository_defined_as_string(self):
        n = Npm2Deb('ipaddr.js')
        self.assertEqual(n.upstream_repo_url,
                         'https://github.com/whitequark/ipaddr.js')

    def test_install_bin(self):
        n = Npm2Deb('mocha')
        n.create_base_debian()
        n.create_links()
        line = self._get_debfile_line('links', 'mocha')
        self.assertEqual(line, 'usr/lib/nodejs/mocha/bin/mocha usr/bin/mocha')

    def test_write_tests(self):
        n = Npm2Deb('debug')
        n.create_base_debian()
        n.create_tests()
        line = self._get_debfile_line('tests/control', 'node-debug')
        self.assertEqual(line, 'Depends: node-debug')
        line = self._get_debfile_line('tests/require', 'debug')
        self.assertEqual(line, """node -e "require('debug');\"""")

    # Issues fixed
    def test_issue_10(self):
        n = Npm2Deb('lastfm')
        n.create_base_debian()
        n.create_control()

    def test_parse_name(self):
        n = Npm2Deb('jade@1.11.0')
        self.assertEqual(n.name, "jade")
        x = Npm2Deb('@ava/write-file-atomic')
        self.assertEqual(x.name, '@ava/write-file-atomic')
        y = Npm2Deb('@types/jquery@2.0.49')
        self.assertEqual(y.name, '@types/jquery')
        z = Npm2Deb('@types/jquery')
        self.assertEqual(z.name, '@types/jquery')



if __name__ == '__main__':
    unittest.main()



from subprocess import getstatusoutput as _getstatusoutput
from subprocess import Popen as _Popen
from subprocess import PIPE as _PIPE
import codecs as _codecs
import locale as _locale
import os as _os
from pathlib import Path as _Path

from npm2deb import templates as _templates

DEBUG_LEVEL = 0
# files starting with this strings will not be included in debian/install
IGNORED_FILES = [
    '.',  # dotfiles
    'readme',  # readme
    'history',
    'changelog',  # history files
    'license',
    'copyright',
    'licence',  # legal files
    'gruntfile',
    'gulpfile',
    'makefile',  # buid system files
    'karma.conf',
    'bower.json',
    'test'  # test files
]


def debug(level, msg):
    if level <= DEBUG_LEVEL:
        print(" debug [%s] - %s" % (level, msg))


def get_npm_version(module_name):
    return _getstatusoutput(
        'npm view "%s" version' % module_name)[1].split('\n')[-2].strip()


def is_ignored(filename):
    filename = filename.lower()
    for pattern in IGNORED_FILES:
        if filename.startswith(pattern):
            return True

    return False


def get_template(filename):
    result = None
    if filename == 'control':
        result = _templates.CONTROL
    elif filename == 'copyright':
        result = _templates.COPYRIGHT
    elif filename == 'rules':
        result = _templates.RULES
    elif filename == 'wnpp':
        result = _templates.WNPP
    return result


def get_watch(which):
    return _templates.WATCH.get(which) or _templates.WATCH.get('npmregistry')


def get_upstream_metadata(which):
    return _templates.METADATA.get(which)


def get_gbp_conf():
    return _templates.GBPCONF


def get_license(license):
    result = None
    name = license.lower().replace('-', '')
    args = {}
    args['name'] = license
    if name.startswith('gpl2'):
        result = _templates.LICENSES['GPL-2']
    elif name.startswith('gpl3'):
        result = _templates.LICENSES['GPL-3']
    elif name.startswith('lgpl2'):
        result = _templates.LICENSES['LGPL-2']
    elif name.startswith('lgpl3'):
        result = _templates.LICENSES['LGPL-3']
    elif name.startswith('mit'):
        result = _templates.LICENSES['MIT']
    elif name.startswith('expat'):
        result = _templates.LICENSES['Expat']
    elif name.startswith('bsd4'):
        result = _templates.LICENSES['BSD-4-clause'] % args
    elif name.startswith('bsd2'):
        result = _templates.LICENSES['BSD-2-clause'] % args
    elif name.startswith('bsd'):
        result = _templates.LICENSES['BSD-3-clause'] % args
    elif name.startswith('artistic'):
        result = _templates.LICENSES['Artistic']
    elif name.startswith('apache'):
        result = _templates.LICENSES['Apache']
    elif name.startswith('isc'):
        result = _templates.LICENSES['ISC']
    else:
        result = 'FIX_ME specify a license, see: ' \
                 'https://www.debian.org/doc/packaging-manuals/' \
                 'copyright-format/1.0/#license-specification'
    return result


def change_dir(dir):
    debug(2, "moving to directory %s" % dir)
    try:
        _os.chdir(dir)
    except OSError as oserror:
        raise OSError("OSError [%d]: %s at %s" %
                      (oserror.errno, oserror.strerror, oserror.filename))


def create_debian_file(filename, content):
    create_file("debian/%s" % filename, content)


def create_file(filename, content):
    debug(2, "creating file %s" % filename)
    content = u'%s' % content
    if len(content) > 0 and content[-1] != '\n':
        content += '\n'
    with _codecs.open(filename, 'w', 'utf-8') as writer:
        writer.write(content)


def create_dir(dir):
    debug(2, "creating directory %s" % dir)
    try:
        path = _Path(dir)
        path.mkdir(parents=True)
    except OSError as oserror:
        raise OSError("Error: directory %s already exists." % oserror.filename)


def parse_name(name):
    parts = name.rpartition('@')
    if parts[0] == '':
        if parts[1] == '@':
            return '@'+parts[2], parts[0]
        return parts[2], parts[0]
    return parts[0], parts[2]


def debianize_name(name):
    return name.replace('_', '-').replace('@', '').replace('/', '-').lower()


def get_latest_debian_standards_version():
    standards_version = _getstatusoutput(
        "rmadison -u debian -s sid debian-policy | cut -d'|' -f2| cut -d'.' -f 1,2,3"
    )[1]
    return standards_version.strip()


def get_npmjs_homepage(name):
    return 'https://npmjs.com/package/' + name


# taken from https://github.com/pallets/click/blob/master/click/_unicodefun.py


def verify_python3_env():
    """Ensures that the environment is good for unicode on Python 3."""
    try:
        fs_enc = _codecs.lookup(_locale.getpreferredencoding()).name
    except Exception:
        fs_enc = 'ascii'
    if fs_enc != 'ascii':
        return

    extra = ''
    if _os.name == 'posix':
        rv = _Popen(
            ['locale', '-a'], stdout=_PIPE, stderr=_PIPE).communicate()[0]
        good_locales = set()
        has_c_utf8 = False

        # Make sure we're operating on text here.
        if isinstance(rv, bytes):
            rv = rv.decode('ascii', 'replace')

        for line in rv.splitlines():
            locale = line.strip()
            if locale.lower().endswith(('.utf-8', '.utf8')):
                good_locales.add(locale)
                if locale.lower() in ('c.utf8', 'c.utf-8'):
                    has_c_utf8 = True

        extra += '\n\n'
        if not good_locales:
            extra += (
                'Additional information: on this system no suitable UTF-8\n'
                'locales were discovered.  This most likely requires resolving\n'
                'by reconfiguring the locale system.')
        elif has_c_utf8:
            extra += (
                'This system supports the C.UTF-8 locale which is recommended.\n'
                'You might be able to resolve your issue by exporting the\n'
                'following environment variables:\n\n'
                '    export LC_ALL=C.UTF-8\n'
                '    export LANG=C.UTF-8')
        else:
            extra += (
                'This system lists a couple of UTF-8 supporting locales that\n'
                'you can pick from.  The following suitable locales were\n'
                'discovered: %s') % ', '.join(sorted(good_locales))

        bad_locale = None
        for locale in _os.environ.get('LC_ALL'), _os.environ.get('LANG'):
            if locale and locale.lower().endswith(('.utf-8', '.utf8')):
                bad_locale = locale
            if locale is not None:
                break
        if bad_locale is not None:
            extra += (
                '\n\nnpm2deb discovered that you exported a UTF-8 locale\n'
                'but the locale system could not pick up from it because\n'
                'it does not exist.  The exported locale is "%s" but it\n'
                'is not supported') % bad_locale

    raise RuntimeError('npm2deb will abort further execution because Python 3 '
                       'was configured to use ASCII as encoding for the '
                       'environment.' + extra)



#!/usr/bin/python3

from distutils.core import setup
from distutils.command.install_scripts import install_scripts
from npm2deb import VERSION
import shutil


class remove_extension(install_scripts):
    def run(self):
        install_scripts.run(self)
        for script in self.get_outputs():
            if script.endswith(".py"):
                shutil.move(script, script[:-3])


setup(name='npm2deb',
      version=VERSION,
      author='Leo Iannacone',
      author_email='l3on@ubuntu.com',
      description='A script to make faster and easier packaging nodejs modules',
      url='https://github.com/LeoIannacone/npm2deb',
      license='GNU GPL-3',
      scripts=['npm2deb.py'],
      packages=['npm2deb'],
      data_files=[
          ('share/man/man1', ['man/npm2deb.1']),
          ('share/doc/npm2deb', ['README.md', 'AUTHORS']),
          ('share/bash-completion/completions', [
              'etc/bash_completion.d/npm2deb'
          ])
      ],
      cmdclass={"install_scripts": remove_extension},
      )



from json import loads as _parseJSON
from datetime import datetime as _datetime
from dateutil import tz as _tz
from shutil import rmtree as _rmtree
from urllib.request import urlopen as _urlopen
from subprocess import getstatusoutput as _getstatusoutput
from subprocess import call as _call
import os as _os
import re as _re
import tarfile

from npm2deb import utils, templates
from npm2deb.mapper import Mapper

VERSION = '0.3.0'
DEBHELPER = 11
STANDARDS_VERSION = '4.3.0'


class Npm2Deb(object):
    def __init__(self, module_name=None, args={}):
        if not module_name and 'node_module' not in args:
            raise ValueError('You must specify a module_name')
        if module_name:
            self.name, self.version = utils.parse_name(module_name)
        elif 'node_module' in args:
            self.name, self.version = utils.parse_name(args['node_module'])
        self.json = None
        self.args = args
        self.homepage = None
        self.upstream_description = None
        self.upstream_author = None
        self.upstream_license = None
        self.upstream_version = None
        self.upstream_repo_url = None
        self.debian_license = "FIX_ME debian license"
        self.debian_standards = STANDARDS_VERSION
        self.debian_debhelper = DEBHELPER
        self.noclean = False
        self.upstream_watch = False
        if args:
            if 'upstream_license' in args and args['upstream_license']:
                self.upstream_license = args['upstream_license']
            if 'upstream_author' in args and args['upstream_author']:
                self.upstream_author = args['upstream_author']
            if 'upstream_homepage' in args and args['upstream_homepage']:
                self.homepage = args['upstream_homepage']
            if 'debian_license' in args and args['debian_license']:
                self.debian_license = args['debian_license']
            if 'standards_version' in args and args['standards_version']:
                self.debian_standards = args['standards_version']
            if 'debhelper' in args:
                self.debian_debhelper = args['debhelper']
            if 'noclean' in args:
                self.noclean = args['noclean']

        self.read_package_info()
        self.debian_name = 'node-%s' % utils.debianize_name(self.name)
        self.debian_author = 'FIX_ME debian author'
        if 'DEBFULLNAME' in _os.environ and 'DEBEMAIL' in _os.environ:
            self.debian_author = "%s <%s>" % \
                (_os.environ['DEBFULLNAME'], _os.environ['DEBEMAIL'])
        elif 'DEBFULLNAME' in _os.environ and 'EMAIL' in _os.environ:
            self.debian_author = "%s <%s>" % \
                (_os.environ['DEBFULLNAME'], _os.environ['EMAIL'])
        self.debian_dest = "usr/lib/nodejs/%s" % self.name
        self.date = _datetime.now(_tz.tzlocal())

    def start(self):
        self.download()
        utils.change_dir(self.debian_name)
        self.create_base_debian()
        self.create_tests()
        self.create_rules()
        self.create_changelog()
        self.create_copyright()
        self.create_control()
        self.create_docs()
        self.create_install()
        self.create_links()
        self.create_dirs()
        self.create_examples()
        self.create_watch()
        self.create_upstream_metadata()
        self.create_gbp_conf()
        self.create_manpages()
        if not self.noclean:
            self.clean()
        utils.change_dir('..')
        self.create_itp_bug()

    def initiate_build(self, saved_path):
        """
        Try building deb package after creating required files using start().
        'uscan', 'uupdate' and 'dpkg-buildpackage' are run if debian/watch is OK.
        """
        uscan_info = self.test_uscan()
        if uscan_info[0] == 0:
            self.run_uscan()
            self.run_uupdate()

            new_dir = '%s-%s' % (self.debian_name, self.upstream_version)
            utils.change_dir('../%s' % new_dir)
            self.run_buildpackage()
            self.edit_changelog()

            debian_path = "%s/%s/debian" % (self.name, new_dir)
            print('\nRemember, your new source directory is %s/%s' %
                  (self.name, new_dir))

        else:
            debian_path = "%s/%s/debian" % (self.name, self.debian_name)

        print("""
This is not a crystal ball, so please take a look at auto-generated files.\n
You may want fix first these issues:\n""")

        utils.change_dir(saved_path)
        _call(
            '/bin/grep --color=auto FIX_ME -r %s/*' % debian_path, shell=True)
        _call(
            '/bin/grep --color=auto FIX_ME -r -H %s/*_itp.mail' % self.name,
            shell=True)

        if uscan_info[0] != 0:
            print(
                "\nUse uscan to get orig source files. Fix debian/watch and then run\
                    \n$ uscan --download-current-version\n")

        if self.upstream_watch:
            print("""
*** Warning ***\nUsing npmregistry to download npm dist tarballs, because upstream
git repo is missing tags. Its better to ask upstream to tag their releases
instead of using npm dist tarballs as dist tarballs may contain pre built files
and may not include tests.\n""")

    def edit_changelog(self):
        """
        To remove extra line '* New upstream release'
        from debian/changelog
        """
        _call(
            "sed -i '/* New upstream release/d' debian/changelog", shell=True)

    def run_buildpackage(self):
        print("\nBuilding the binary package")
        _call('dpkg-source -b .', shell=True)
        _call('rm -rf .gitignore .git* .travis.yml', shell=True)
        _call('dpkg-buildpackage', shell=True)
        # removing auto generated temporary files
        _call('debian/rules clean', shell=True)

    def run_uupdate(self):
        print('\nCreating debian source package...')
        _call(
            'uupdate -b -f --upstream-version %s' % self.upstream_version,
            shell=True)

    def run_uscan(self):
        print('\nDownloading source tarball file using debian/watch file...')
        _call(
            'uscan --download-version %s' % self.upstream_version, shell=True)

    def test_uscan(self):
        info = _getstatusoutput('uscan --watchfile "debian/watch" '
                                '--package "{}" '
                                '--upstream-version 0 '
                                '--download-version {} '
                                '--no-download'.format(self.debian_name,
                                                       self.upstream_version))
        return info

    def create_itp_bug(self):
        utils.debug(1, "creating wnpp bug template")
        utils.create_file('%s_itp.mail' % self.debian_name, self.get_ITP())

    def clean(self):
        utils.debug(1, "cleaning directory")
        for filename in _os.listdir('.'):
            if filename != 'debian':
                if _os.path.isdir(filename):
                    _rmtree(filename)
                else:
                    _os.remove(filename)

    def create_manpages(self):
        if 'man' not in self.json:
            return
        mans = self.json['man']
        if not isinstance(mans, (list, tuple)):
            mans = [mans]
        paths = [_os.path.normpath(manpath) for manpath in mans]
        utils.create_debian_file('manpages', "\n".join(paths))

    def create_watch(self):
        args = {}
        args['debian_name'] = self.debian_name
        args['dversionmangle'] = 'auto'
        args['url'] = self.upstream_repo_url
        args['module'] = self.name
        args['remodule'] = _re.sub(r'\@', r'\@', self.name)
        args['modulename'] = _re.sub(r'.*/', r'', self.name)
        try:
            if self.upstream_repo_url.find('github') >= 0:
                utils.debug(1, 'Found GitHub url')
                content = utils.get_watch('github') % args
            elif self.upstream_repo_url.find('gitlab') >= 0:
                utils.debug(1, 'Found GitLab url')
                content = utils.get_watch('gitlab') % args
            else:
                # if not supported, got to npmregistry
                raise ValueError

            utils.create_debian_file('watch', content)
            # test watch with uscan, raise exception if status is not 0
            uscan_info = self.test_uscan()

            if uscan_info[0] != 0:
                utils.debug(1, 'Uscan failed, switching to npmregistry')
                raise ValueError

        except ValueError:
            self.upstream_watch = True
            content = utils.get_watch('npmregistry') % args
            utils.create_debian_file('watch', content)

    def create_upstream_metadata(self):
        args = {}
        args['url'] = _re.sub(r'^git\+', '', self.upstream_repo_url)
        args['module'] = self.name
        if self.upstream_repo_url.find('github') >= 0:
            content = utils.get_upstream_metadata('github') % args
            utils.create_dir("debian/upstream")
            utils.create_debian_file('upstream/metadata', content)
        elif self.upstream_repo_url.find('gitlab') >= 0:
            content = utils.get_upstream_metadata('gitlab') % args
            utils.create_dir("debian/upstream")
            utils.create_debian_file('upstream/metadata', content)

    def create_gbp_conf(self):
        content = utils.get_gbp_conf()
        utils.create_debian_file('gbp.conf', content)

    def create_examples(self):
        if _os.path.isdir('examples'):
            content = 'examples/*'
            utils.create_debian_file('examples', content)

    def create_dirs(self):
        if _os.path.isdir('bin'):
            content = 'usr/bin'
            utils.create_debian_file('dirs', content)

    def create_links(self):
        links = []
        dest = self.debian_dest
        if 'bin' in self.json:
            for script in self.json['bin']:
                orig = _os.path.normpath(self.json['bin'][script])
                links.append("%s/%s usr/bin/%s" % (dest, orig, script))
        if len(links) > 0:
            content = '\n'.join(links)
            utils.create_debian_file('links', content)

    def create_install(self):
        content = ''
        libs = _os.listdir()
        # remove debian directory
        libs.remove('debian')

        for filename in libs:
            if not utils.is_ignored(filename):
                content += "%s %s/\n" % (filename, self.debian_dest)
        utils.create_debian_file('install', content)

    def create_docs(self):
        docs = []
        if 'readmeFilename' in self.json and self.json['readmeFilename']:
            docs.append(self.json['readmeFilename'])
        else:
            for name in _os.listdir('.'):
                if name.lower().startswith('readme'):
                    docs.append(name)
                    break
        if len(docs) > 0:
            content = '\n'.join(docs)
            utils.create_debian_file('docs', content)

    def create_control(self):
        args = {}
        args['Source'] = self.debian_name
        args['Uploaders'] = self.debian_author
        args['debhelper'] = self.debian_debhelper
        args['Standards-Version'] = self.debian_standards
        args['Homepage'] = self.homepage
        args['Vcs-Git'] = 'https://salsa.debian.org/' + \
                          'js-team/%s.git' \
                          % self.debian_name
        args['Vcs-Browser'] = 'https://salsa.debian.org/' + \
                              'js-team/%s' \
                              % self.debian_name
        args['Package'] = self.debian_name
        args['Depends'] = self._get_Depends()
        args['Description'] = 'FIX_ME write the Debian package description'
        args['upstream_description'] = self.upstream_description
        template = utils.get_template('control')
        utils.create_debian_file('control', template % args)

    def create_copyright(self):
        args = {}
        args['upstream_name'] = self.name
        args['source'] = self.homepage
        args['upstream_date'] = self.date.year
        args['upstream_author'] = self.upstream_author
        args['upstream_contact'] = self.upstream_author
        if 'bugs' in self.json and 'url' in self.json['bugs']:
            args['upstream_contact'] = self.json['bugs']['url']
        args['upstream_license_name'] = self.upstream_license
        if self.debian_license and \
                self.upstream_license != self.debian_license:
            args['upstream_license'] = "\nLicense: %s" % \
                utils.get_license(self.upstream_license)
        else:
            args['upstream_license'] = ''  # do not insert same license twice
        args['debian_date'] = self.date.year
        args['debian_author'] = self.debian_author
        args['debian_license_name'] = self.debian_license
        args['debian_license'] = "License: %s" % \
            utils.get_license(self.debian_license)
        template = utils.get_template('copyright')
        utils.create_debian_file('copyright', template % args)

    def create_changelog(self):
        args = {}
        args['debian_author'] = self.debian_author
        args['debian_name'] = self.debian_name
        args['version'] = self.upstream_version
        args['date'] = self.date.strftime('%a, %d %b %Y %X %z')
        file_content = templates.CHANGELOG % args
        utils.create_debian_file("changelog", file_content)

    def create_rules(self):
        args = {}
        args['overrides'] = ''
        for filename in _os.listdir('.'):
            if filename.lower().startswith('history'):
                args['overrides'] += "override_dh_installchangelogs:\n" + \
                                     "\tdh_installchangelogs -k %s\n" % filename
                break
        content = utils.get_template('rules') % args
        utils.create_debian_file("rules", content)
        _os.system('chmod +x debian/rules')

    def create_tests(self):
        utils.create_dir("debian/tests")
        args = {}
        args['name'] = self.name
        args['debian_name'] = self.debian_name

    def create_base_debian(self):
        utils.debug(1, "creating debian files")
        utils.create_dir("debian")
        utils.create_dir("debian/source")
        utils.create_debian_file("source/format", "3.0 (quilt)\n")
        utils.create_debian_file("compat", self.debian_debhelper)

    def read_package_info(self):
        data = None
        name_is = None
        if _re.match("^(http://|https://)", self.name):
            utils.debug(1, "reading json - opening url %s" % self.name)
            data = _urlopen(self.name).read().decode('utf-8')
            name_is = 'url'

        elif _os.path.isfile(self.name):
            utils.debug(1, "reading json - opening file %s" % self.name)
            with open(self.name, 'r') as fd:
                data = fd.read()
            name_is = 'file'

        else:
            name_is = 'npm'
            utils.debug(1, "reading json - calling npm view %s@%s" % (self.name, self.version))
            info = _getstatusoutput('npm view "%s@%s" --json 2>/dev/null' %
                                    (self.name, self.version))
            # if not status 0, raise expection
            if info[0] != 0:
                info = _getstatusoutput('npm view "%s@%s" --json' % (self.name, self.version))
                exception = 'npm reports errors about %s module:\n' % self.name
                exception += info[1]
                raise ValueError(exception)
            if not info[1]:
                exception = 'npm returns empty json for %s module' % self.name
                raise ValueError(exception)
            data = info[1]

        try:
            self.json = _parseJSON(data)
        except ValueError as value_error:
            # if error during parse, try to fail graceful
            if str(value_error) == 'Expecting value: line 1 column 1 (char 0)':
                if name_is != 'npm':
                    raise ValueError("Data read from %s "
                                     "is not in a JSON format." % name_is)
                versions = []
                for line in data.split('\n'):
                    if _re.match(r"^[a-z](.*)@[0-9]", line):
                        version = line.split('@')[1]
                        versions.append(version)
                if len(versions) > 0:
                    exception = "More than one version found. "\
                        "Please specify one of:\n %s" % '\n '.join(versions)
                    raise ValueError(exception)
                else:
                    raise value_error
            else:
                raise value_error

        self.name = self.json['name']
        self._get_json_author()
        self._get_json_repo_url()
        self._get_json_homepage()
        self._get_json_description()
        self._get_json_version()
        self._get_json_license()

    def download(self):
        utils.debug(1, "downloading %s@%s tarball from npm registry" % (self.name, self.version))
        info = _getstatusoutput('npm pack "%s@%s"' % (self.name, self.version))
        if info[0] is not 0:
            exception = "Error downloading package %s@s\n" % (self.name, self.version)
            exception += info[1]
            raise ValueError(exception)
        tarball_file = info[1].split('\n')[-1]
        tarball = tarfile.open(tarball_file)
        # get the root directory name
        root_dir = tarball.getnames()[0]
        # extract root directory name if memberfile contains '/'
        index_of_slash = root_dir.find('/')
        if index_of_slash != -1:
            root_dir = root_dir[:index_of_slash]

        tarball.extractall()
        tarball.close()

        # remove tarball file
        _os.remove(tarball_file)

        if root_dir is not self.debian_name:
            utils.debug(2, "renaming %s to %s" % (root_dir, self.debian_name))
            # rename extracted directory
            _os.rename(root_dir, self.debian_name)

    def get_ITP(self):
        args = {}
        args['debian_author'] = self.debian_author
        args['debian_name'] = self.debian_name
        args['upstream_author'] = self.upstream_author
        args['homepage'] = self.homepage
        args['description'] = 'FIX_ME write the Debian package description'
        args['upstream_description'] = self.upstream_description
        args['version'] = self.upstream_version
        args['license'] = self.upstream_license
        content = utils.get_template('wnpp')
        return content % args

    def _get_json_license(self):
        if not self.upstream_license:
            self.upstream_license = "FIX_ME upstream license"
            license_name = None
            if 'licenses' in self.json:
                license_name = self.json['licenses']
            elif 'license' in self.json:
                license_name = self.json['license']
            if license_name:
                if isinstance(license_name, list):
                    license_name = license_name[0]
                if isinstance(license_name, dict):
                    license_name = license_name['type']
                if license_name.lower() == "mit":
                    license_name = "Expat"
                self.upstream_license = license_name
        if self.debian_license is None or \
           self.debian_license.find('FIX_ME') >= 0:
            self.debian_license = self.upstream_license

    def _get_json_version(self):
        if 'version' in self.json:
            self.upstream_version = self.json['version']
        else:
            self.upstream_version = 'FIX_ME version'

    def _get_json_description(self):
        if 'description' in self.json:
            self.upstream_description = self.json['description']
        else:
            self.upstream_description = 'FIX_ME no upstream package description'

    def _get_json_author(self):
        if self.upstream_author:
            return
        result = 'FIX_ME upstream author'
        if 'author' in self.json:
            author = self.json['author']
            if isinstance(author, str):
                result = author
            elif isinstance(author, dict):
                if 'name' in author and 'email' in author:
                    result = "%s <%s>" % (author['name'], author['email'])
                elif 'name' in author:
                    result = author['name']
        self.upstream_author = result

    def _get_json_repo_url(self):
        result = 'FIX_ME repo url'
        url = None
        if 'repository' in self.json:
            repository = self.json['repository']
            if isinstance(repository, str):
                url = repository
            elif isinstance(repository, dict) and 'url' in repository:
                url = repository['url']

            if not url:
                pass  # repository field is not in expected format
            elif url.startswith('git') or (isinstance(repository, dict)
                                           and 'type' in repository
                                           and repository['type'] == 'git'):
                if url.find('github') >= 0:
                    tmp = self._get_github_url_from_git(url)
                    if tmp:
                        result = tmp
                else:
                    utils.debug(1, ("raw URL: %s" % url))
                    url = _re.sub(r'^git.*?@(.*?):', r'https://\1/', url)
                    url = _re.sub(r'^.*?(?:ssh|git).*?://(?:.*?\@)?', 'https://', url)
                    url = _re.sub(r'\.git$', '', url)
                    result = url
                    utils.debug(1, ("https URL: %s" % url))
            else:
                result = url
        self.upstream_repo_url = result

    def _get_json_homepage(self):
        if self.homepage:
            return

        if 'homepage' in self.json:
            self.homepage = self.json['homepage']
        elif self.upstream_repo_url and not \
                self.upstream_repo_url.find('FIX_ME') >= 0:
            self.homepage = self.upstream_repo_url
        else:
            self.homepage = utils.get_npmjs_homepage(self.name)

    def _get_Depends(self):
        depends = ['nodejs (>= 6)']
        mapper = Mapper.get_instance()
        if 'dependencies' in self.json:
            dependencies = self.json['dependencies']
            for dep in dependencies:
                name = mapper.get_debian_package(dep)['name']
                if not name:
                    name = 'node-%s' % dep
                    mapper.append_warning(
                        'error', dep, 'dependency %s '
                        'not in debian' % (name))
                version = dependencies[dep]
                if version:
                    version = version.lower().replace('~', '').replace(
                        '^', '').replace('.x', '.0')
                    if version[0].isdigit():
                        version = '>= %s' % version
                    elif version == '*' or version == 'latest':
                        version = None
                if version:
                    dep_debian = "%s (%s)" % (name, version)
                else:
                    dep_debian = name
                depends.append(dep_debian)

        return '\n , '.join(depends)

    def _get_github_url_from_git(self, url):
        result = _getstatusoutput(
            "node -e "
            "\"console.log(require('github-url-from-git')"
            "('%s'));\"" % url)[1]
        if result == 'undefined':
            result = None
        return result
